import sys
import time
from datetime import datetime
from .html_builder import *

class Table(Tag):
    def __init__(self,attrs,options):
        idx=attrs['id']
        super().__init__('div')
        if attrs.get('class'):
            attrs['class']+=' _col_ lztbl _nslct_'
        else:
            attrs['class']=' _col_ lztbl _nslct_'
        self.setAttributes(attrs)
        attrs['onwheel']=f"LAZYTABLE.onwheel(event,'{idx}');"\
                         f"LAZYTABLE.populate('{idx}')"
        self.tableBody=self.row({'onmousedown':f"LAZYTABLE.mousedownBody(event,'{idx}')",
                                 'ondblclick':f"LAZYTABLE.makeEditable(event,'{idx}')",
                                 'onmouseleave':f"LAZYTABLE.mouseleaveBody(event,'{idx}')"})
        if options.get('radios'):
            self.inputCol=self.tableBody.column({'class':'lztblinpcol'})
            th=self.inputCol.Div({'text':'⦿','class':'lztblth'})
            self.inputTdCol=self.inputCol.column({'class':'lztblinpcolinr lztblcol'})
            td=self.inputTdCol.Div({'class':'lztbltd'})
            td.Input({'type':'radio','name':'rdos'+idx,
                      'onchange':f"LAZYTABLE.onclickinput(event,'{idx}');"
                                 f"LAZYTABLE.populate('{idx}')"})
        elif options.get('checks'):
            self.inputCol=self.tableBody.column({'class':'lztblinpcol'})
            th=self.inputCol.row({'text':'✔','class':'lztblth',
                             'onclick':f"LAZYTABLE.selectAll('{idx}');"
                                       f"LAZYTABLE.populate('{idx}')"})
            self.inputTdCol=self.inputCol.column({'class':'lztblinpcolinr lztblcol'})
            td=self.inputTdCol.Div({'class':'lztbltd'})
            td.Input({'type':'checkbox',
                      'onmousedown':f"LAZYTABLE.mousedownCheckbox(event,'{idx}')",
                      'onclick':f"LAZYTABLE.onclickinput(event,'{idx}');"
                                 f"LAZYTABLE.populate('{idx}')"})
        self.tableBodyInner=self.tableBody.row({'class':'lztblbdyinner'})
        for c,h in enumerate(options['headers']):
            bodyCol=self.tableBodyInner.column({'class':'lztbltdcoloutout'})
            if options.get('colWidths'):
                # self.bodyCol.attrs['class']+=' lztblsetwdcol'
                bodyCol.attrs['style']={'width':str(options['colWidths'][c])+'px'}
            th=bodyCol.row({'class':'lztblth'})
            th.P({'text':'▼','class':'lztblsorttgl',
                  'onclick':f"LAZYTABLE.sort(event,'{idx}');"
                            f"LAZYTABLE.populate('{idx}')"})
            th.Input({'type':'text','placeholder':h,'class':'lztblsrch',
                      'onkeydown':f"LAZYTABLE.filter(event,'{idx}');"
                                  f"LAZYTABLE.populate('{idx}')"})
            bodyTdColOuter=bodyCol.column({'class':'lztbltdcolout'})
            if options.get('colWidths'):
                bodyTdColOuter.attrs['class']+=' lztblsetwdcolout'
            else:
                bodyTdColOuter.attrs['class']+=' lztblautowdcolout'
            bodyTdCol=bodyTdColOuter.column({'class':'lztbltdcol lztblcol'})
            if options.get('colWidths'):
                bodyTdCol.attrs['class']+=' lztblsetwdcol'
            else:
                bodyTdCol.attrs['class']+=' lztblautowdcol'
                # bodyTdCol.attrs['style']={'width':str(options['colWidths'][c])+'px'}
            td=bodyTdCol.Div({'class':'lztbltd'})
        ytrack=self.tableBody.column({'id':'ytrk_'+idx,'class':'lztblytrk'})
        ythumb=ytrack.Div({'id':'ythmb_'+idx,'class':'lztblythmb',
                           'onmousedown':f"LAZYTABLE.onmousedownThumb(event,'{idx}')"})
        self.infoRow=self.row({'style':{'position':'relative'}})
        self.infoRow.P({'text':'⚙️','onclick':f"LAZYTABLE.toggleOptionsPanel(event,'{idx}')"})
        col=self.infoRow.popup({'class':'lztbloptpnl',
             'onmouseleave':"this.style.display='none'"})
        for h in options['headers']:
            col.Label({'text':h}).Input({'type':'checkbox',
                 'onchange':f"LAZYTABLE.toggleColumnVisibility(event,'{idx}')"},['checked'])
        self.infoRow.P({'text':'∑'})
        self.infoRow.P({'text':'0','id':'ttl'+idx})
        self.infoRow.P({'text':':&nbsp;'})
        self.infoRow.P({'text':'0','id':'rgstrt'+idx})
        self.infoRow.P({'text':'~'})
        self.infoRow.P({'text':'0','id':'rgend'+idx})
        self.infoRow.P({'text':'&nbsp;✔'})
        self.infoRow.P({'text':'0','id':'chkd'+idx})
        js=self.Script()
        js.addLines('{')
        js.addVar('opts',options)
        js.addVar('tidx',idx)
        js.addLines(
            "LAZYTABLE.setMeta(tidx,opts)"
        )
        js.addLines('}')



from .date_time import *
from .db_objects import *
from .dividers import *
from .html_builder import *
from .lazy_table import *
from .lazy_tree import *
from .mysql_query_builder import *
from .read_creds import *
from .rich_text_editor import *
from router_A import *
from router_B import *
from .tabs import *


import os
import json
def read():
    dbUser=os.getenv('DBUSER')
    dbPw=os.getenv('DBPW')
    cwd=os.getcwd()
    print(cwd)
    fp=os.path.join(cwd,'creds.json')
    if os.path.isfile(fp):
        fl=open(fp,'r')
        creds=fl.read()
        fl.close()
        creds=json.loads(creds)
        return creds
    else:
        creds={
            "user":dbUser,
            "password":dbPw,
        }
        return creds


import sys
import time
from datetime import datetime,timedelta
import requests
import psutil
import socket
import multiprocessing as mp
from aiohttp import web
from .db_info import *
from .mysql_query_builder import *
from .read_creds import read
CREDS=read()
HUBDBI=getDbInstance(user=CREDS['localhost']['user'],
                     password=CREDS['localhost']['password'])
def get_idlest():
    t1=HUBDBI.web_ms.routers
    t2=HUBDBI.web_ms.user_router_binding
    t3=HUBDBI.network_traffic.t
    now=datetime.now()
    sq=Select(t1.port).str()
    routerCount={}
    crs=HUBDBI.query(sq)
    for (port,) in crs:
        routerCount[port]=0
    sq=Select(t3.port,t3.upload,t3.download)\
        .where(t3.time.ge(now-timedelta(minutes=5))).str()
    crs=HUBDBI.query(sq)
    for port,upload,download in crs:
        routerCount[port]=routerCount.get(port) or 0
        routerCount[port]+=upload+download
    mn=10e10
    mnPort=None
    for port in routerCount:
        if routerCount[port]<mn:
            mn=routerCount[port]
            mnPort=port
    return mnPort
def handle_client(client_socket,addr):
    t1=HUBDBI.web_ms.routers
    t2=HUBDBI.web_ms.user_router_binding
    ip=addr[0]
    now=datetime.now()
    sq=Select(t2.port).where(t2.ip.eq(ip)\
        .And(t2.time.ge(now-timedelta(minutes=60)))).str()
    crs=HUBDBI.query(sq)
    port=None
    for (port,) in crs: pass
    if not port:
        port=get_idlest()
    sq=Insert(t2.ip,t2.port,t2.time).values([
        ip,
        port,
        SQL.now()
    ]).on_duplicate_update(t2.port,t2.time).str()
    HUBDBI.query(sq)
    message = client_socket.recv(1024).decode()
    print(f"[+] Received: {message[:100]}...")
    fullUrl=message.split("HTTP")[0]
    if 'GET' in fullUrl:
        fullUrl=fullUrl.split('GET')[1].strip()
    elif 'POST' in fullUrl:
        fullUrl=fullUrl.split('POST')[1].strip()
    url=f'http://127.0.0.1:{port}{fullUrl}'
    print(url)
    client_socket.sendall(f"HTTP/1.1 200 OK\r\nContent-type: text/html\r\n\r\n"
       f"<script>window.location='{url}'</script>\r\n\r\n".encode())
    client_socket.close()
if __name__=='__main__':
    print(f'[+]',CREDS)
    if len(sys.argv)>1:
        bind_port=int(sys.argv[1])
    else:
        bind_port=5000
    bind_ip="0.0.0.0"
    print(f"[+] Listening on port {bind_ip} : {bind_port}")
    server=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    server.bind((bind_ip,bind_port))
    # we tell the server to start listening with
    # a maximum backlog of connections set to 5
    server.listen(1000)
    while True:
        # When a client connects we receive the
        # client socket into the client variable, and
        # the remote connection details into the addr variable
        client, addr = server.accept()
        print(f"[+] Accepted connection from: {addr[0]}:{addr[1]}")
        #spin up our client thread to handle the incoming data
        # client_handler = threading.Thread(target=handle_client, args=(client,))
        client_handler = mp.Process(target=handle_client,args=(client,addr))
        client_handler.start()



import json
from collections import deque
class Tag:
    def __init__(self,tagName,singleton=False):
        self.tagName=tagName
        self.singleton=singleton
        self.children=[]
    def setAttributes(self,attrs=None,sattrs=None):
        self.attrs=attrs
        self.sattrs=sattrs
    def appendChild(self,child):
        self.children.append(child)
        child.parent=child.parentElement=self
        return child
    def nextSibling(self):
        parent=self.parent
        children=parent.children
        ix=children.index(self)
        if ix<len(children)-1:
            return children[ix+1]
        return None
    def previousSibling(self):
        parent=self.parent
        children=parent.children
        ix=children.index(self)
        if ix>0:
            return children[ix-1]
        return None
    def before(self,element):
        parent=self.parent
        children=parent.children
        ix=children.index(self)
        children.insert(ix,element)
        return element
    def after(self,element):
        parent=self.parent
        children=parent.children
        ix=children.index(self)
        children.insert(ix+1,element)
        return element
    def _appendStr(self,pre,depth=0):
        # if depth:
        #     pre.append('\n')
        # for d in range(depth):
        #     pre.append('\t')
        pre.append('<')
        pre.append(self.tagName)
        if self.attrs:
            for k,v in self.attrs.items():
                if k=='text': continue
                elif k=='style':
                    pre.append(' style="')
                    for k2,v2 in v.items():
                        pre.append(k2)
                        pre.append(':')
                        pre.append(str(v2))
                        pre.append(';')
                    pre.append('"')
                elif k=='class':
                    pre.append(' ')
                    pre.append(k)
                    pre.append('=')
                    pre.append('"')
                    pre.append(str(v))
                    pre.append('"')
                else:
                    pre.append(' ')
                    pre.append(k)
                    pre.append('=')
                    pre.append('"')
                    pre.append(str(v))
                    pre.append('"')
        if self.sattrs:
            for sa in self.sattrs:
                pre.append(' ')
                pre.append(sa)
        pre.append('>')
        if self.attrs and 'text' in self.attrs and self.tagName!='label':
            pre.append(self.attrs['text'])
        for child in self.children:
            if child.tagName=='script':
                pre.append(child.str())
            elif child.tagName=='style':
                pre.append(child.str())
            else:
                child._appendStr(pre,depth+1)
        if self.attrs and 'text' in self.attrs and self.tagName=='label':
            pre.append(self.attrs['text'])
        if not self.singleton:
            # pre.append('\n')
            # for d in range(depth):
            #     pre.append('\t')
            pre.append(f'</{self.tagName}>')
    def str(self):
        depth=0
        pre=[]
        self._appendStr(pre,depth)
        return ''.join(pre)
    def column(self,attrs=None):
        if not attrs: attrs={}
        if attrs.get('class'):
            attrs['class']+=' _col_'
        else:
            attrs['class']='_col_'
        return self.appendChild(Div(attrs))
    def row(self,attrs=None):
        if not attrs: attrs={}
        if attrs.get('class'):
            attrs['class']+=' _row_'
        else:
            attrs['class']='_row_'
        return self.appendChild(Div(attrs))
    def popup(self,attrs=None):
        if not attrs: attrs={}
        if attrs.get('class'):
            attrs['class']+=' _popup_ _col_'
        else:
            attrs['class']='_popup_ _col_'
        if attrs.get('style'):
            attrs['style']['display']='none'
        else:
            attrs['style']={'display':'none'}
        return self.appendChild(Div(attrs))
    def tooltip(self,attrs=None):
        if not attrs: attrs={}
        if attrs.get('class'):
            attrs['class']+=' _tltptxt_'
        else:
            attrs['class']='_tltptxt_ '
        if self.attrs.get('class'):
            self.attrs['class']+=' _tltp_'
        else:
            self.attrs['class']='_tltp_'
        return self.appendChild(Div(attrs))
    def Area(self,attrs=None,sattrs=None):
        return self.appendChild(Area(attrs,sattrs))
    def Base(self,attrs=None,sattrs=None):
        return self.appendChild(Base(attrs,sattrs))
    def Body(self,attrs=None,sattrs=None):
        return self.appendChild(Body(attrs,sattrs))
    def Br(self,attrs=None,sattrs=None):
        return self.appendChild(Br(attrs,sattrs))
    def Col(self,attrs=None,sattrs=None):
        return self.appendChild(Col(attrs,sattrs))
    def Command(self,attrs=None,sattrs=None):
        return self.appendChild(Command(attrs,sattrs))
    def Embed(self,attrs=None,sattrs=None):
        return self.appendChild(Embed(attrs,sattrs))
    def Hr(self,attrs=None,sattrs=None):
        return self.appendChild(Hr(attrs,sattrs))
    def Img(self,attrs=None,sattrs=None):
        return self.appendChild(Img(attrs,sattrs))
    def Input(self,attrs=None,sattrs=None):
        return self.appendChild(Input(attrs,sattrs))
    def Keygen(self,attrs=None,sattrs=None):
        return self.appendChild(Keygen(attrs,sattrs))
    def Link(self,attrs=None,sattrs=None):
        return self.appendChild(Link(attrs,sattrs))
    def Meta(self,attrs=None,sattrs=None):
        return self.appendChild(Meta(attrs,sattrs))
    def Param(self,attrs=None,sattrs=None):
        return self.appendChild(Param(attrs,sattrs))
    def Source(self,attrs=None,sattrs=None):
        return self.appendChild(Source(attrs,sattrs))
    def Track(self,attrs=None,sattrs=None):
        return self.appendChild(Track(attrs,sattrs))
    def Wbr(self,attrs=None,sattrs=None):
        return self.appendChild(Wbr(attrs,sattrs))
    def A(self,attrs=None,sattrs=None):
        return self.appendChild(A(attrs,sattrs))
    def Abbr(self,attrs=None,sattrs=None):
        return self.appendChild(Abbr(attrs,sattrs))
    def Address(self,attrs=None,sattrs=None):
        return self.appendChild(Address(attrs,sattrs))
    def Article(self,attrs=None,sattrs=None):
        return self.appendChild(Article(attrs,sattrs))
    def Aside(self,attrs=None,sattrs=None):
        return self.appendChild(Aside(attrs,sattrs))
    def Audio(self,attrs=None,sattrs=None):
        return self.appendChild(Audio(attrs,sattrs))
    def B(self,attrs=None,sattrs=None):
        return self.appendChild(B(attrs,sattrs))
    def Bdi(self,attrs=None,sattrs=None):
        return self.appendChild(Bdi(attrs,sattrs))
    def Bdo(self,attrs=None,sattrs=None):
        return self.appendChild(Bdo(attrs,sattrs))
    def Blockquote(self,attrs=None,sattrs=None):
        return self.appendChild(Blockquote(attrs,sattrs))
    def Body(self,attrs=None,sattrs=None):
        return self.appendChild(Body(attrs,sattrs))
    def Button(self,attrs=None,sattrs=None):
        return self.appendChild(Button(attrs,sattrs))
    def Canvas(self,attrs=None,sattrs=None):
        return self.appendChild(Canvas(attrs,sattrs))
    def Caption(self,attrs=None,sattrs=None):
        return self.appendChild(Caption(attrs,sattrs))
    def Cite(self,attrs=None,sattrs=None):
        return self.appendChild(Cite(attrs,sattrs))
    def Colgroup(self,attrs=None,sattrs=None):
        return self.appendChild(Colgroup(attrs,sattrs))
    def Data(self,attrs=None,sattrs=None):
        return self.appendChild(Data(attrs,sattrs))
    def Datalist(self,attrs=None,sattrs=None):
        return self.appendChild(Datalist(attrs,sattrs))
    def Dd(self,attrs=None,sattrs=None):
        return self.appendChild(Dd(attrs,sattrs))
    def Del(self,attrs=None,sattrs=None):
        return self.appendChild(Del(attrs,sattrs))
    def Details(self,attrs=None,sattrs=None):
        return self.appendChild(Details(attrs,sattrs))
    def Dfn(self,attrs=None,sattrs=None):
        return self.appendChild(Dfn(attrs,sattrs))
    def Dialog(self,attrs=None,sattrs=None):
        return self.appendChild(Dialog(attrs,sattrs))
    def Div(self,attrs=None,sattrs=None):
        return self.appendChild(Div(attrs,sattrs))
    def Dl(self,attrs=None,sattrs=None):
        return self.appendChild(Dl(attrs,sattrs))
    def Dt(self,attrs=None,sattrs=None):
        return self.appendChild(Dt(attrs,sattrs))
    def Em(self,attrs=None,sattrs=None):
        return self.appendChild(Em(attrs,sattrs))
    def Fieldset(self,attrs=None,sattrs=None):
        return self.appendChild(Fieldset(attrs,sattrs))
    def Figcaption(self,attrs=None,sattrs=None):
        return self.appendChild(Figcaption(attrs,sattrs))
    def Figure(self,attrs=None,sattrs=None):
        return self.appendChild(Figure(attrs,sattrs))
    def Footer(self,attrs=None,sattrs=None):
        return self.appendChild(Footer(attrs,sattrs))
    def Form(self,attrs=None,sattrs=None):
        return self.appendChild(Form(attrs,sattrs))
    def H1(self,attrs=None,sattrs=None):
        return self.appendChild(H1(attrs,sattrs))
    def H2(self,attrs=None,sattrs=None):
        return self.appendChild(H2(attrs,sattrs))
    def H3(self,attrs=None,sattrs=None):
        return self.appendChild(H3(attrs,sattrs))
    def H4(self,attrs=None,sattrs=None):
        return self.appendChild(H4(attrs,sattrs))
    def H5(self,attrs=None,sattrs=None):
        return self.appendChild(H5(attrs,sattrs))
    def H6(self,attrs=None,sattrs=None):
        return self.appendChild(H6(attrs,sattrs))
    def Head(self,attrs=None,sattrs=None):
        return self.appendChild(Head(attrs,sattrs))
    def Header(self,attrs=None,sattrs=None):
        return self.appendChild(Header(attrs,sattrs))
    def Hgroup(self,attrs=None,sattrs=None):
        return self.appendChild(Hgroup(attrs,sattrs))
    def Html(self,attrs=None,sattrs=None):
        return self.appendChild(Html(attrs,sattrs))
    def I(self,attrs=None,sattrs=None):
        return self.appendChild(I(attrs,sattrs))
    def Iframe(self,attrs=None,sattrs=None):
        return self.appendChild(Iframe(attrs,sattrs))
    def Ins(self,attrs=None,sattrs=None):
        return self.appendChild(Ins(attrs,sattrs))
    def Kbd(self,attrs=None,sattrs=None):
        return self.appendChild(Kbd(attrs,sattrs))
    def Label(self,attrs=None,sattrs=None):
        return self.appendChild(Label(attrs,sattrs))
    def Legend(self,attrs=None,sattrs=None):
        return self.appendChild(Legend(attrs,sattrs))
    def Li(self,attrs=None,sattrs=None):
        return self.appendChild(Li(attrs,sattrs))
    def Main(self,attrs=None,sattrs=None):
        return self.appendChild(Main(attrs,sattrs))
    def Map(self,attrs=None,sattrs=None):
        return self.appendChild(map(attrs,sattrs))
    def Mark(self,attrs=None,sattrs=None):
        return self.appendChild(Mark(attrs,sattrs))
    def Menu(self,attrs=None,sattrs=None):
        return self.appendChild(Menu(attrs,sattrs))
    def Meter(self,attrs=None,sattrs=None):
        return self.appendChild(Meter(attrs,sattrs))
    def Nav(self,attrs=None,sattrs=None):
        return self.appendChild(Nav(attrs,sattrs))
    def Noscript(self,attrs=None,sattrs=None):
        return self.appendChild(Noscript(attrs,sattrs))
    def Object(self,attrs=None,sattrs=None):
        return self.appendChild(Object(attrs,sattrs))
    def Ol(self,attrs=None,sattrs=None):
        return self.appendChild(Ol(attrs,sattrs))
    def Optgroup(self,attrs=None,sattrs=None):
        return self.appendChild(Optgroup(attrs,sattrs))
    def Option(self,attrs=None,sattrs=None):
        return self.appendChild(Option(attrs,sattrs))
    def Output(self,attrs=None,sattrs=None):
        return self.appendChild(Output(attrs,sattrs))
    def P(self,attrs=None,sattrs=None):
        return self.appendChild(P(attrs,sattrs))
    def Picture(self,attrs=None,sattrs=None):
        return self.appendChild(Picture(attrs,sattrs))
    def Pre(self,attrs=None,sattrs=None):
        return self.appendChild(Pre(attrs,sattrs))
    def Progress(self,attrs=None,sattrs=None):
        return self.appendChild(Progress(attrs,sattrs))
    def Q(self,attrs=None,sattrs=None):
        return self.appendChild(Q(attrs,sattrs))
    def Rp(self,attrs=None,sattrs=None):
        return self.appendChild(Rp(attrs,sattrs))
    def Rt(self,attrs=None,sattrs=None):
        return self.appendChild(Rt(attrs,sattrs))
    def Ruby(self,attrs=None,sattrs=None):
        return self.appendChild(Ruby(attrs,sattrs))
    def S(self,attrs=None,sattrs=None):
        return self.appendChild(S(attrs,sattrs))
    def Script(self,attrs=None,sattrs=None):
        return self.appendChild(Script(attrs,sattrs))
    def Search(self,attrs=None,sattrs=None):
        return self.appendChild(Search(attrs,sattrs))
    def Section(self,attrs=None,sattrs=None):
        return self.appendChild(Section(attrs,sattrs))
    def Select(self,attrs=None,sattrs=None):
        return self.appendChild(Select(attrs,sattrs))
    def Small(self,attrs=None,sattrs=None):
        return self.appendChild(Small(attrs,sattrs))
    def Span(self,attrs=None,sattrs=None):
        return self.appendChild(Span(attrs,sattrs))
    def Strong(self,attrs=None,sattrs=None):
        return self.appendChild(Strong(attrs,sattrs))
    def Style(self,attrs=None,sattrs=None):
        return self.appendChild(Style(attrs,sattrs))
    def Sub(self,attrs=None,sattrs=None):
        return self.appendChild(Sub(attrs,sattrs))
    def Summary(self,attrs=None,sattrs=None):
        return self.appendChild(Summary(attrs,sattrs))
    def Sup(self,attrs=None,sattrs=None):
        return self.appendChild(Sup(attrs,sattrs))
    def Svg(self,attrs=None,sattrs=None):
        return self.appendChild(Svg(attrs,sattrs))
    def Table(self,attrs=None,sattrs=None):
        return self.appendChild(Table(attrs,sattrs))
    def Tbody(self,attrs=None,sattrs=None):
        return self.appendChild(Tbody(attrs,sattrs))
    def Td(self,attrs=None,sattrs=None):
        return self.appendChild(Td(attrs,sattrs))
    def Template(self,attrs=None,sattrs=None):
        return self.appendChild(Template(attrs,sattrs))
    def Textarea(self,attrs=None,sattrs=None):
        return self.appendChild(Textarea(attrs,sattrs))
    def Tfoot(self,attrs=None,sattrs=None):
        return self.appendChild(Tfoot(attrs,sattrs))
    def Th(self,attrs=None,sattrs=None):
        return self.appendChild(Th(attrs,sattrs))
    def Thead(self,attrs=None,sattrs=None):
        return self.appendChild(Thead(attrs,sattrs))
    def Time(self,attrs=None,sattrs=None):
        return self.appendChild(Time(attrs,sattrs))
    def Title(self,attrs=None,sattrs=None):
        return self.appendChild(Title(attrs,sattrs))
    def Tr(self,attrs=None,sattrs=None):
        return self.appendChild(Tr(attrs,sattrs))
    def U(self,attrs=None,sattrs=None):
        return self.appendChild(U(attrs,sattrs))
    def Ul(self,attrs=None,sattrs=None):
        return self.appendChild(ul(attrs,sattrs))
    def Var(self,attrs=None,sattrs=None):
        return self.appendChild(Var(attrs,sattrs))
    def Video(self,attrs=None,sattrs=None):
        return self.appendChild(Video(attrs,sattrs))
class Doctype(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('!DOCTYPE html',True)
        self.setAttributes(attrs,sattrs)
class Area(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('area',True)
        self.setAttributes(attrs,sattrs)
class Base(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('base',True)
        self.setAttributes(attrs,sattrs)
class Br(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('br',True)
        self.setAttributes(attrs,sattrs)
class Col(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('col',True)
        self.setAttributes(attrs,sattrs)
class Command(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('command',True)
        self.setAttributes(attrs,sattrs)
class Embed(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('embed',True)
        self.setAttributes(attrs,sattrs)
class Hr(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('hr',True)
        self.setAttributes(attrs,sattrs)
class Img(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('img',True)
        self.setAttributes(attrs,sattrs)
class Input(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('input',True)
        self.setAttributes(attrs,sattrs)
class Keygen(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('keygen',True)
        self.setAttributes(attrs,sattrs)
class Link(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('link',True)
        self.setAttributes(attrs,sattrs)
class Meta(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('meta',True)
        self.setAttributes(attrs,sattrs)
class Param(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('param',True)
        self.setAttributes(attrs,sattrs)
class Source(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('source',True)
        self.setAttributes(attrs,sattrs)
class Track(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('track',True)
        self.setAttributes(attrs,sattrs)
class Wbr(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('wbr',True)
        self.setAttributes(attrs,sattrs)
class A(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('a')
        self.setAttributes(attrs,sattrs)
class Abbr(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('abbr')
        self.setAttributes(attrs,sattrs)
class Address(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('address')
        self.setAttributes(attrs,sattrs)
class Article(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('article')
        self.setAttributes(attrs,sattrs)
class Aside(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('aside')
        self.setAttributes(attrs,sattrs)
class Audio(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('audio')
        self.setAttributes(attrs,sattrs)
class B(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('b')
        self.setAttributes(attrs,sattrs)
class Bdi(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('bdi')
        self.setAttributes(attrs,sattrs)
class Bdo(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('bdo')
        self.setAttributes(attrs,sattrs)
class Blockquote(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('blockquote')
        self.setAttributes(attrs,sattrs)
class Body(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('body')
        self.setAttributes(attrs,sattrs)
class Button(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('button')
        self.setAttributes(attrs,sattrs)
class Canvas(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('canvas')
        self.setAttributes(attrs,sattrs)
class Caption(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('caption')
        self.setAttributes(attrs,sattrs)
class Cite(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('cite')
        self.setAttributes(attrs,sattrs)
class Colgroup(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('colgroup')
        self.setAttributes(attrs,sattrs)
class Data(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('data')
        self.setAttributes(attrs,sattrs)
class Datalist(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('datalist')
        self.setAttributes(attrs,sattrs)
class Dd(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('dd')
        self.setAttributes(attrs,sattrs)
class Del(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('del')
        self.setAttributes(attrs,sattrs)
class Details(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('details')
        self.setAttributes(attrs,sattrs)
class Dfn(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('dfn')
        self.setAttributes(attrs,sattrs)
class Dialog(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('dialog')
        self.setAttributes(attrs,sattrs)
class Div(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('div')
        self.setAttributes(attrs,sattrs)
class Dl(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('dl')
        self.setAttributes(attrs,sattrs)
class Dt(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('dt')
        self.setAttributes(attrs,sattrs)
class Em(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('em')
        self.setAttributes(attrs,sattrs)
class Fieldset(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('fieldset')
        self.setAttributes(attrs,sattrs)
class Figcaption(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('figcaption')
        self.setAttributes(attrs,sattrs)
class Figure(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('figure')
        self.setAttributes(attrs,sattrs)
class Footer(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('footer')
        self.setAttributes(attrs,sattrs)
class Form(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('form')
        self.setAttributes(attrs,sattrs)
class H1(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('h1')
        self.setAttributes(attrs,sattrs)
class H2(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('h1')
        self.setAttributes(attrs,sattrs)
class H3(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('h1')
        self.setAttributes(attrs,sattrs)
class H4(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('h1')
        self.setAttributes(attrs,sattrs)
class H5(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('h1')
        self.setAttributes(attrs,sattrs)
class H6(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('h1')
        self.setAttributes(attrs,sattrs)
class Head(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('head')
        self.setAttributes(attrs,sattrs)
class Header(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('header')
        self.setAttributes(attrs,sattrs)
class Hgroup(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('hgroup')
        self.setAttributes(attrs,sattrs)
class Html(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('html')
        self.setAttributes(attrs,sattrs)
class I(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('i')
        self.setAttributes(attrs,sattrs)
class Iframe(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('iframe')
        self.setAttributes(attrs,sattrs)
class Ins(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('ins')
        self.setAttributes(attrs,sattrs)
class Kbd(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('kbd')
        self.setAttributes(attrs,sattrs)
class Label(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('label')
        self.setAttributes(attrs,sattrs)
class Legend(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('legend')
        self.setAttributes(attrs,sattrs)
class Li(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('li')
        self.setAttributes(attrs,sattrs)
class Main(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('main')
        self.setAttributes(attrs,sattrs)
class Map(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('map')
        self.setAttributes(attrs,sattrs)
class Mark(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('mark')
        self.setAttributes(attrs,sattrs)
class Menu(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('menu')
        self.setAttributes(attrs,sattrs)
class Meter(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('meter')
        self.setAttributes(attrs,sattrs)
class Nav(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('nav')
        self.setAttributes(attrs,sattrs)
class Noscript(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('noscript')
        self.setAttributes(attrs,sattrs)
class Object(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('object')
        self.setAttributes(attrs,sattrs)
class Ol(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('ol')
        self.setAttributes(attrs,sattrs)
class Optgroup(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('optgroup')
        self.setAttributes(attrs,sattrs)
class Option(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('option')
        self.setAttributes(attrs,sattrs)
class Output(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('output')
        self.setAttributes(attrs,sattrs)
class P(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('p')
        self.setAttributes(attrs,sattrs)
class Picture(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('picture')
        self.setAttributes(attrs,sattrs)
class Pre(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('pre')
        self.setAttributes(attrs,sattrs)
class Progress(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('progress')
        self.setAttributes(attrs,sattrs)
class Q(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('q')
        self.setAttributes(attrs,sattrs)
class Rp(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('rp')
        self.setAttributes(attrs,sattrs)
class Rt(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('rt')
        self.setAttributes(attrs,sattrs)
class Ruby(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('ruby')
        self.setAttributes(attrs,sattrs)
class S(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('s')
        self.setAttributes(attrs,sattrs)
class Samp(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('samp')
        self.setAttributes(attrs,sattrs)
class Script(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('script')
        self.setAttributes(attrs,sattrs)
        self.rawlines=[]
    def addLines(self,*lines):
        self.rawlines.extend(lines)
    def addVar(self,varname,value):
        if type(value)==str:
            self.rawlines.append(f"let {varname}='{value}'")
        elif type(value) in (int,float):
            self.rawlines.append(f"let {varname}={value}")
        else:
            self.rawlines.append(f"let {varname}={json.dumps(value)}")
    def addJsFile(self,filepath):
        with open(filepath,'r',encoding='utf8') as fl:
            self.rawlines.append(fl.read())
    def str(self):
        pre=[]
        pre.append('<script')
        if self.attrs:
            for k,v in self.attrs.items():
                pre.append(' ')
                pre.append(k)
                pre.append('=')
                pre.append('"')
                pre.append(str(v))
                pre.append('"')
        pre.append('>')
        for line in self.rawlines:
            pre.append(line)
            pre.append('\n')
        pre.append('</script>')
        return ''.join(pre)
class Search(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('search')
        self.setAttributes(attrs,sattrs)
class Section(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('section')
        self.setAttributes(attrs,sattrs)
class Select(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('select')
        self.setAttributes(attrs,sattrs)
class Small(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('small')
        self.setAttributes(attrs,sattrs)
class Span(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('span')
        self.setAttributes(attrs,sattrs)
class Strong(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('strong')
        self.setAttributes(attrs,sattrs)
class Style(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('style')
        self.setAttributes(attrs,sattrs)
        self.css={}
        self.cssFiles=[]
    def addCss(self,selector,cssDict):
        self.css[selector]=cssDict
    def addCssFile(self,filepath):
        with open(filepath,'r') as fl:
            css=fl.read()
            self.cssFiles.append(css)
    def str(self):
        pre=[]
        pre.append('<style>')
        for selector,cssDict in self.css.items():
            pre.append(selector)
            pre.append('{')
            for k,v in cssDict.items():
                pre.append(k)
                pre.append(':')
                pre.append(str(v))
                pre.append(';')
            pre.append('}')
        for css in self.cssFiles:
            pre.append(css)
        pre.append('</style>')
        return ''.join(pre)
class Sub(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('Sub')
        self.setAttributes(attrs,sattrs)
class Summary(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('summary')
        self.setAttributes(attrs,sattrs)
class Sup(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('sup')
        self.setAttributes(attrs,sattrs)
class Svg(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('svg')
        self.setAttributes(attrs,sattrs)
class Table(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('table')
        self.setAttributes(attrs,sattrs)
class Tbody(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('tbody')
        self.setAttributes(attrs,sattrs)
class Td(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('td')
        self.setAttributes(attrs,sattrs)
class Template(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('template')
        self.setAttributes(attrs,sattrs)
class Textarea(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('textarea')
        self.setAttributes(attrs,sattrs)
class Tfoot(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('tfoot')
        self.setAttributes(attrs,sattrs)
class Th(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('th')
        self.setAttributes(attrs,sattrs)
class Thead(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('thead')
        self.setAttributes(attrs,sattrs)
class Time(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('time')
        self.setAttributes(attrs,sattrs)
class Title(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('title')
        self.setAttributes(attrs,sattrs)
class Tr(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('tr')
        self.setAttributes(attrs,sattrs)
class U(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('u')
        self.setAttributes(attrs,sattrs)
class Ul(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('ul')
        self.setAttributes(attrs,sattrs)
class Var(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('var')
        self.setAttributes(attrs,sattrs)
class Video(Tag):
    def __init__(self,attrs=None,sattrs=None):
        super().__init__('video')
        self.setAttributes(attrs,sattrs)
class HtmlDoc:
    def __init__(self):
        self.doctype=Doctype()
        self.html=self.doctype.appendChild(Html())
        self.head=self.html.appendChild(Head())
        js=self.head.Script()
        js.addLines(
            "window.print=console.log",
            # "window.True=true",
            # "window.False=false",
        )
        js.addJsFile('./basic_funcs.js')
        css=self.head.Style()
        css.addCssFile('./css.css')
        self.body=self.html.appendChild(Body())
    def str(self):
        return self.doctype.str()
if __name__=='__main__':
    print(P({'text':'asdf'}).str())
    print(P().str())
    print(HtmlDoc().str())
    pass


import socket

target_host = "127.0.0.1"
target_port = 5000

#creating socket object
client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#connecting the client
client.connect((target_host, target_port))
#sending data
client.send("this is a message\r\n\r\n".encode())
#receiving the data
response = client.recv(4096)
print(response.decode())


import sys
import time
from datetime import datetime
from .html_builder import *

class Tree(Tag):
    def __init__(self,attrs,options):
        idx=attrs['id']
        super().__init__('div')
        if attrs.get('class'):
            attrs['class']+=' _col_ lztre _nslct_'
        else:
            attrs['class']=' _col_ lztre _nslct_'
        self.setAttributes(attrs)
        self.filterCol=self.column({'class':'lztrefltrcol'})
        self.filterCol.attrs['style']={}
        if options.get('filterColumnHeight'):
            self.filterCol.attrs['style']['height']=str(options['filterColumnHeight'])+'px'
        for h in options['headers']:
            row=self.filterCol.row({'class':'lztrfltrrow'})
            row.Input({'type':'text','placeholder':f'🔍 {h}','header':h,'class':'lztrefltr',
               'onkeydown':f"LAZYTREE.filter(event,'{idx}')"})
        self.bodyRow=self.row({'class':'lztrebodyrow',
            'onwheel':f"LAZYTREE.onwheel(event,'{idx}');"
                      f"LAZYTREE.populate('{idx}')"})
        self.bodyCol=self.bodyRow.column({'class':'lztrebodycol',
              'onmousedown':f"LAZYTREE.onmousedownBody(event,'{idx}')"})
        self.yTrack=self.bodyRow.column({'class':'lztreytrk'})
        self.yThumb=self.yTrack.Div({'class':'lztreythmb',
            'onmousedown':f"LAZYTREE.onmousedownThumb(event,'{idx}');"})
        self.infoRow=self.row()
        self.infoRow.P({'text':'⚙️','onclick':f"LAZYTREE.toggleOptionsPanel(event,'{idx}')"})
        col=self.infoRow.popup({'class':'lztreoptpnl',
             'onmouseleave':"this.style.display='none'"})
        for h in options['headers']:
            col.Label({'text':h,
               'onmousedown':f"LAZYTREE.onmousedownColVisInput(event,'{idx}')",
           }).Input({'type':'checkbox',
               'onchange':f"LAZYTREE.toggleColumnVisibility(event,'{idx}')"},['checked'])
        self.infoRow.P({'text':'✔'})
        self.infoRow.P({'text':'0','id':'lztrechkd'+idx})
        js=self.Script()
        js.addLines('{')
        js.addVar('tidx',idx)
        js.addVar('opts',options)
        js.addLines(
            f"LAZYTREE.setMeta(tidx,opts)",
        )
        js.addLines('}')


import sys
import time
from datetime import datetime
import requests
import psutil
import socket
HOSTNAME=socket.gethostname()
HOST=socket.gethostbyname(HOSTNAME)
import os
PID=os.getpid()
PPID=os.getppid()
print(HOSTNAME,HOST,PPID,PID)
from .db_info import *
from .mysql_query_builder import *
from .read_creds import read
CREDS=read()
HUBDBI=getDbInstance(user=CREDS['localhost']['user'],
                     password=CREDS['localhost']['password'])
# if len(sys.argv)<2:
#     raise Exception('enter port')
# bind_port=int(sys.argv[1])
import socket
import threading
import traceback
import multiprocessing as mp
def get_user_info(USER_INFO):
    t1=HUBDBI.hub.users
    t2=HUBDBI.hub.user_perms
    sq1=Select(t1.uid,t1.name,t1.ip,t1.expiry).str()
    sq2=Select(t2.uid,t2.perm).str()
    while True:
        user_perms={}
        try:
            crs=HUBDBI.query(sq2,cursor='dict')
        except Exception:
            traceback.print_exc()
            time.sleep(2)
            continue
        for d in crs:
            uid=d['uid']
            user_perms[uid]=user_perms.get(uid) or set()
            user_perms[uid].add(d['perm'])
        try:
            crs=HUBDBI.query(sq1,cursor='dict')
        except Exception:
            traceback.print_exc()
            time.sleep(2)
            continue
        for d in crs:
            d['perms']=user_perms.get(d['uid']) or set()
            USER_INFO[d['ip']]=d
            USER_INFO[d['uid']]=d
        time.sleep(2)
def get_routing_info(ROUTING_INFO,ROUTING_COUNTS,USER_ROUTING_INFO):
    t1=HUBDBI.web_ms.ms
    t2=HUBDBI.web_ms.routing
    t3=HUBDBI.web_ms.user_routing
    sq2=Select(t2.ms,t2.host,t2.port,t1.requiredPerm).str()
    sq3=Select(t3.uid,t3.ms,t3.host,t3.port,t3.time)\
        .where(t3.time.ge(datetime.now()-timedelta(minutes=60))).str()
    while True:
        try:
            crs=HUBDBI.query(sq2,cursor='dict')
        except Exception:
            traceback.print_exc()
            time.sleep(2)
            continue
        routing={}
        for d in crs:
            url=d['ms']
            routing[url]=routing.get(url) or {'requiredPerm':d['requiredPerm'],'addresses':[]}
            hp=(d['host'],d['port'])
            routing[url]['addresses'].append(hp)
            ROUTING_COUNTS[hp]=0
        for url in routing:
            ROUTING_INFO[url]=routing[url]
        try:
            crs=HUBDBI.query(sq3,cursor='dict')
        except Exception:
            traceback.print_exc()
            time.sleep(2)
            continue
        for d in crs:
            hp=(d['host'],d['port'])
            ROUTING_COUNTS[hp]+=1
            USER_ROUTING_INFO[(d['uid'],d['ms'])]=d
        time.sleep(2)
def get_idlest(ROUTING_INFO,ROUTING_COUNTS,url):
    mn=10e10
    mnHp=None
    for hp in ROUTING_INFO[url]['addresses']:
        if hp=='meta':continue
        if ROUTING_COUNTS[hp]<mn:
            mn=ROUTING_COUNTS[hp]
            mnHp=hp
    return {'host':mnHp[0],'port':mnHp[1]}
def logger(qu):
    t1=HUBDBI.usage_log.t
    t2=HUBDBI.web_ms.user_routing
    while True:
        kwa=qu.get()
        now=datetime.now()
        ym=(now.year,now.month)
        if kwa['type']=='usage_log':
            sq=Insert(t1.uid,t1.url,t1.fullUrl,t1.time,t1.
                      responseSize,t1.responseTime,
                      t1.error,t1.body).values([
                kwa['uid'],
                kwa['url'],
                kwa['fullUrl'],
                SQL.now(),
                kwa['responseSize'],
                kwa['responseTime'],
                kwa['error'],
                kwa['body']
            ]).str(ym)
        elif kwa['type']=='user_routing':
            sq=Insert(t2.uid,t2.ms,t2.host,t2.port,t2.time).values([
                kwa['uid'],
                kwa['url'],
                kwa['host'],
                kwa['port'],
                SQL.now(),
            ]).on_duplicate_update(t2.host,t2.port,t2.time).str()
        else:
            print(f"unknown logger type {kwa['type']}")
            continue
        try:
            HUBDBI.query(sq)
        except Exception as e:
            print(e)
def get_network_usage(PORT):
    UPDATE_DELAY=2
    io=psutil.net_io_counters()
    bytes_sent,bytes_recv=io.bytes_sent,io.bytes_recv
    t1=HUBDBI.network_traffic.t
    while True:
        now=datetime.now()
        ym=(now.year,now.month)
        time.sleep(UPDATE_DELAY)
        io_2=psutil.net_io_counters()
        sent=io_2.bytes_sent-bytes_sent
        recv=io_2.bytes_recv-bytes_recv
        print(f"Upload: {sent}, Download: {recv}")
        bytes_sent,bytes_recv=io_2.bytes_sent,io_2.bytes_recv
        sq=Insert(t1.hostname,t1.host,t1.port,t1.time,t1.pid,t1.upload,t1.download).values([
            HOSTNAME,
            HOST,
            PORT,
            SQL.now(),
            PPID,
            sent,
            recv
        ]).str(ym)
        HUBDBI.query(sq)
def handle_client(client_socket,addr,USER_INFO,ROUTING_INFO,ROUTING_COUNTS,
                  USER_ROUTING_INFO,LOGGING_QU):
    timer=time.time()
    message = client_socket.recv(1024).decode()
    body=None
    if 'POST' in message:
        body=[]
        while True:
            chunk=client_socket.recv(1024).decode()
            body.apppend(chunk)
            if chunk[-4:]=='\r\n\r\n':
                break
        body=''.join(body)
    print(f"[+] Received: {message[:100]}...")
    fullUrl=message.split("HTTP")[0]
    respCode=None
    resp=None
    get=False
    post=False
    if 'GET' in fullUrl:
        get=True
        fullUrl=fullUrl.split('GET')[1].strip()
    elif 'POST' in fullUrl:
        post=True
        fullUrl=fullUrl.split('POST')[1].strip()
    else:
        respCode=400
        resp='unregistered HTTP method'
    print(fullUrl)
    url='/'.join(fullUrl.split('/')[:2])
    user_info=USER_INFO.get(addr[0])
    print(addr[0])
    uid=user_info['uid']
    print(uid,datetime.now(),user_info['expiry'])
    if respCode is None:
        if fullUrl=='/favicon.ico':
            respCode=400
            resp='no favicon'
        elif user_info is None:
            respCode=401
            resp='no users found'
        elif datetime.now()>user_info['expiry']:
            respCode=401
            resp='user authorization expired'
        elif url not in ROUTING_INFO:
            respCode=400
            resp=f'no routing info for {fullUrl}'
        elif ROUTING_INFO[url]['requiredPerm'] \
            and ROUTING_INFO[url]['requiredPerm'] not in user_info['perms']:
            respCode=401
            resp='user does not have required permissions'
        else:
            ri=USER_ROUTING_INFO.get((uid,url))
            if not ri: ri=get_idlest(ROUTING_INFO,ROUTING_COUNTS,url)
            USER_ROUTING_INFO[(uid,url)]=ri
            LOGGING_QU.put({
                'type':'user_routing',
                'uid':uid,
                'url':url,
                'host':ri['host'],
                'port':ri['port'],
            })
            rl=f"http://{ri['host']}:{ri['port']}{fullUrl}"
            print(rl,'HERE')
            if get:
                resp=requests.get(rl).text
            elif post:
                resp=requests.post(rl,data=body).text
            respCode=200
    #sending back the packet
    resp=(f"HTTP/1.1 {respCode} OK\r\nContent-type: text/html\r\n\r\n"
       f"{resp}\r\n\r\n")
    client_socket.sendall(resp.encode())
    client_socket.close()
    timeTaken=time.time()-timer
    print('time taken',round(timeTaken,4),'s')
    LOGGING_QU.put({
        'type':'usage_log',
        'uid':uid,
        'url':url,
        'fullUrl':fullUrl,
        'responseSize':len(resp),
        'responseTime':timeTaken,
        'error':resp if respCode!=200 else None,
        'body':message,
    })
if __name__=='__main__':
    print(f'[+]',CREDS)
    if len(sys.argv)>1:
        bind_port=int(sys.argv[1])
    else:
        bind_port=5001
    bind_ip="0.0.0.0"
    print(f"[+] Listening on port {bind_ip} : {bind_port}")
    manager=mp.Manager()
    USER_INFO=manager.dict()
    ROUTING_INFO=manager.dict()
    ROUTING_COUNTS=manager.dict()
    USER_ROUTING_INFO=manager.dict()
    LOGGING_QU=mp.Queue()
    # user_info_thread=threading.Thread(target=get_user_info)
    user_info_thread=mp.Process(target=get_user_info,args=(USER_INFO,))
    user_info_thread.start()
    logger_thread=mp.Process(target=logger,args=(LOGGING_QU,))
    logger_thread.start()
    network_usage_thread=mp.Process(target=get_network_usage,args=(bind_port,))
    network_usage_thread.start()
    routing_info_thread=mp.Process(target=get_routing_info,
                                   args=(ROUTING_INFO,ROUTING_COUNTS,USER_ROUTING_INFO))
    routing_info_thread.start()
    server=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    server.bind((bind_ip,bind_port))
    # we tell the server to start listening with
    # a maximum backlog of connections set to 5
    server.listen(1000)
    while True:
        # When a client connects we receive the
        # client socket into the client variable, and
        # the remote connection details into the addr variable
        client, addr = server.accept()
        print(f"[+] Accepted connection from: {addr[0]}:{addr[1]}")
        #spin up our client thread to handle the incoming data
        # client_handler = threading.Thread(target=handle_client, args=(client,))
        client_handler = mp.Process(target=handle_client,
            args=(client,addr,USER_INFO,ROUTING_INFO,ROUTING_COUNTS,
                  USER_ROUTING_INFO,LOGGING_QU))
        client_handler.start()



import sys
import time
from datetime import datetime
from .html_builder import *

class RichTextEditor(Tag):
    def __init__(self,attrs):
        super().__init__('div')
        idx=attrs['id']
        if attrs.get('class'):
            attrs['class']+=' _col_ rchtxtedtr'
        else:
            attrs['class']=' _col_ rchtxtedtr'
        self.setAttributes(attrs)
        bar=self.row({'style':{'display':'flex','position':'relative',
                               'white-space':'pre','flex-wrap':'wrap'}})
        bar.Button({'text':'B','style':{'font-weight':'bold'},
                    'onclick':f"RichTextEditor.makeBold(event,'{idx}')"})
        bar.Button({'text':'I','style':{'font-style':'italic'},
                    'onclick':f"RichTextEditor.makeItalic(event,'{idx}')"})
        bar.Button({'text':'U','style':{'text-decoration':'underline'},
                    'onclick':f"RichTextEditor.makeUnderlined(event,'{idx}')"})
        bar.Button({'text':'S','style':{'text-decoration':'line-through'},
                    'onclick':f"RichTextEditor.makeLineThrough(event,'{idx}')"})
        button=bar.Button({'text':'S',
                    'onclick':f"RichTextEditor.makeSuperscript(event,'{idx}')"})
        button.Sup({'text':'x'})
        button=bar.Button({'text':'S',
                    'onclick':f"RichTextEditor.makeSubscript(event,'{idx}')"})
        button.Sub({'text':'x'})
        button=bar.Button({'text':'A','style':{'color':'blue'}})
        button.Input({'type':'color','value':'#0000FF','style':{'width':'calc(var(--fsz) * 2)'},
                      'onchange':f"RichTextEditor.changeFgColor(event,'{idx}')"})
        button=bar.Button({'text':'A','style':{'background-color':'blue','color':'white'}})
        button.Input({'type':'color','value':'#0000FF','style':{'width':'calc(var(--fsz) * 2)'},
                      'onchange':f"RichTextEditor.changeBgColor(event,'{idx}')"})
        bar.Button({'text':'➕▦','onclick':f"RichTextEditor.toggleTablePanel(event,'{idx}')"})
        row=bar.row({'class':'rchtxtedtrtblpnl','style':{'display':'none'}})
        row.Input({'type':'number','style':{'width':'30px'},'placeholder':4,'min':1})
        row.P({'text':'X'})
        row.Input({'type':'number','style':{'width':'30px'},'placeholder':4,'min':1})
        bar.Button({'text':'❌▦','onclick':f"RichTextEditor.deleteTable(event,'{idx}')"})
        row.Button({'text':'✔️','onclick':f"RichTextEditor.addTable(event,'{idx}')"})
        bar.Button({'text':'➕▤','onclick':f"RichTextEditor.addRow(event,'{idx}')"})
        bar.Button({'text':'▤➕','onclick':f"RichTextEditor.addRow(event,'{idx}',true)"})
        bar.Button({'text':'❌▤','onclick':f"RichTextEditor.deleteRow(event,'{idx}')"})
        bar.Button({'text':'➕▥','onclick':f"RichTextEditor.addColumn(event,'{idx}')"})
        bar.Button({'text':'▥➕','onclick':f"RichTextEditor.addColumn(event,'{idx}',true)"})
        bar.Button({'text':'❌▥','onclick':f"RichTextEditor.deleteColumn(event,'{idx}')"})
        self.body=self.Div({'class':'rchtxtedtrcont','contenteditable':'true',
           'onkeydown':f"RichTextEditor.keydown(event,'{idx}')",
           'onpaste':f"RichTextEditor.onpaste(event,'{idx}')",
        })


import sys
import time
from datetime import datetime
import requests
from .db_info import *
from .mysql_query_builder import *
from .html_builder import *
from fastapi import FastAPI,Response
import uvicorn
app=FastAPI()
from .lazy_table import Table
from .lazy_tree import Tree
from .rich_text_editor import RichTextEditor
from .tabs import Tabs,TabButtons
from .dividers import HorizontalDivider,VerticalDivider
import random
from string import ascii_letters as letters
@app.get('/home')
async def home():
    doc=HtmlDoc()
    css=doc.head.Style()
    css.addCssFile('./lazy_table.css')
    css.addCssFile('./lazy_tree.css')
    css.addCssFile('./rich_text_editor.css')
    css.addCssFile('./tabs.css')
    css.addCssFile('./dividers.css')
    js=doc.head.Script()
    js.addJsFile('./lazy_table_funcs.js')
    js.addJsFile('./lazy_tree_funcs.js')
    js.addJsFile('./rich_text_editor_funcs.js')
    js.addJsFile('./tabs_funcs.js')
    js.addJsFile('./dividers_funcs.js')
    js.addJsFile('./chart_scatter.js')
    js.addJsFile('./chart_scatter_time.js')
    js.addJsFile('./chart_line.js')
    doc.body.P({'text':'ping received (ms)'})
    row0=doc.body.row()
    col=row0.column({'style':{'height':'200px','width':'200px'}})
    col.appendChild(Table({'id':'table','style':{'width':'100%'}},
                          {'checks':True,'headers':['col1','col2']}))
    js=doc.body.Script()
    ls=[[random.randint(0,1000),random.randint(0,1000)] for _ in range(100000)]
    js.addVar('ls',ls)
    js.addLines(
        "LAZYTABLE.resize('table')",
        "LAZYTABLE.setData('table',ls)",
        "LAZYTABLE.populate('table')",
    )
    row0.appendChild(HorizontalDivider())
    row1=row0.row({'style':{'width':'600px'}})
    col=row1.column({'style':{'min-height':'100%','width':'200px'}})
    col.appendChild(Table({'id':'table2'},{'radios':True,'headers':['col1','col2'],
                    'colWidths':[100,100]}))
    js=doc.body.Script()
    ls=[]
    for _ in range(10000):
        a=''.join([letters[random.randint(0,len(letters)-1)] for _ in range(20)])
        b=''.join([letters[random.randint(0,len(letters)-1)] for _ in range(20)])
        ls.append([a,b])
    js.addVar('ls2',ls)
    js.addLines(
        "LAZYTABLE.resize('table2')",
        "LAZYTABLE.setData('table2',ls2)",
        "LAZYTABLE.populate('table2')",
    )
    row1.appendChild(HorizontalDivider())
    col=row1.column({'style':{'height':'300px','width':'calc(100% - 205px)'}})
    col.appendChild(Tree({'id':'tree'},{'headers':['firstlayer','secondlayer','thirdLayer'],'colorSelectors':True}))
    js=doc.body.Script()
    js.addVar('data',[
        {'firstlayer':'asdf','secondlayer':'1234','thirdLayer':'asdfsacxz'},
        {'firstlayer':'asdf','secondlayer':'1234','thirdLayer':'asdf'},
        {'firstlayer':'asdf','secondlayer':'1234','thirdLayer':'tyterdh'},
        {'firstlayer':'asdf','secondlayer':'7573','thirdLayer':'dgh'},
        {'firstlayer':'asdf','secondlayer':'7573','thirdLayer':'dfghf'},
        {'firstlayer':'asdf','secondlayer':'7573','thirdLayer':'4yrty'},
        {'firstlayer':'asdf','secondlayer':'7573','thirdLayer':'cvmr'},
        {'firstlayer':'qwer','secondlayer':'7fgd573','thirdLayer':'gdhgh'},
        {'firstlayer':'qwer','secondlayer':'7fgd573','thirdLayer':'xg'},
        {'firstlayer':'qwer','secondlayer':'7fgd573','thirdLayer':'fghg'},
        {'firstlayer':'qwer','secondlayer':'36643fga','thirdLayer':'g'},
        {'firstlayer':'qwer','secondlayer':'36643fga','thirdLayer':'gfh'},
        {'firstlayer':'qwer','secondlayer':'36643fga','thirdLayer':'asdfshfgacxz'},
        {'firstlayer':'qwer','secondlayer':'36643fga','thirdLayer':'ththt'},
        {'firstlayer':'qwer','secondlayer':'36643fga','thirdLayer':'vbrrr'},
        {'firstlayer':'zxcv','secondlayer':'7fgd573','thirdLayer':'bbrhehe'},
        {'firstlayer':'zxcv','secondlayer':'7fgd573','thirdLayer':'htetth'},
        {'firstlayer':'zxcv','secondlayer':'7fgd573','thirdLayer':'hfghg'},
        {'firstlayer':'zxcv','secondlayer':'7fgd573','thirdLayer':'ghgfhd'},
        {'firstlayer':'zxcv','secondlayer':'sdfdsfsf','thirdLayer':'drery'},
        {'firstlayer':'zxcv','secondlayer':'sdfdsfsf','thirdLayer':'eruyhhhj'},
        {'firstlayer':'zxcv','secondlayer':'sdfdsfsf','thirdLayer':'htrtt'},
        {'firstlayer':'zxcv','secondlayer':'sdfdsfsf','thirdLayer':'cghth'},
        {'firstlayer':'tyui','secondlayer':'7fgd573','thirdLayer':'cghgh'},
        {'firstlayer':'tyui','secondlayer':'7fgd573','thirdLayer':'eyqerq'},
        {'firstlayer':'tyui','secondlayer':'7fgd573','thirdLayer':'cghgerrt'},
        {'firstlayer':'tyui','secondlayer':'7fgd573','thirdLayer':'qsgsdh'},
        {'firstlayer':'tyui','secondlayer':'sdfdsfsf','thirdLayer':'dhdhcbv'},
        {'firstlayer':'tyui','secondlayer':'sdfdsfsf','thirdLayer':'cvbrwrrer'},
        {'firstlayer':'tyui','secondlayer':'sdfdsfsf','thirdLayer':'dfghdfh'},
        {'firstlayer':'tyui','secondlayer':'sdfdsfsf','thirdLayer':'rytytyt'},
    ])
    js.addLines(
        f"LAZYTREE.setData('tree',data)",
        f"LAZYTREE.populate('tree')",
    )
    row0=doc.body.row({'style':{'height':'300px'}})
    rte=row0.appendChild(RichTextEditor({'id':'rte','style':{},
         'onmousedown':f"RichTextEditor.onmousedownTable(event,'rte')"}))
    rte.body.attrs['text']=('asldkjflskjfsld<div>zxcvzcxzxcvz</div><table><tbody>'
                            '<tr><td>a</td><td>b</td><td>c</td><td>d</td></tr>'
                            '<tr><td>e</td><td>f</td><td>g</td><td>h</td></tr>'
                            '<tr><td>i</td><td>j</td><td>k</td><td>l</td></tr>'
                            '<tr><td>m</td><td>n</td><td>o</td><td>p</td></tr>'
                            '</tbody></table><div>qwerqwerqwr</div>')
    doc.body.appendChild(VerticalDivider())
    col1=doc.body.column({'style':{'margin-top':'10px','width':'300px','height':'150px'}})
    tabNames=['tab1','tab2','tab3']
    col1.appendChild(TabButtons(tabNames))
    tabCont=col1.appendChild(Tabs(len(tabNames)))
    tabCont.children[0].P({'text':'tab 1 content'})
    button=tabCont.children[0].Button({'text':'dummy button'})
    button.tooltip({'text':'this is a dummy button that has a tooltip'})
    tabCont.children[1].P({'text':'asdlkfjsalkdfasdfasdfseryretyrhrthrrtadfasdfasfsadfasasfds'})
    tabCont.children[2].P({'text':'zzxcvzxcvxzcuio'})
    doc.body.appendChild(VerticalDivider())
    row1=doc.body.row({'style':{'width':'600px','height':'300px'}})
    row1.Canvas({'id':'graph1'})
    js=doc.body.Script()
    numPoints=1000
    rainbow=['red','orange','yellow','green','blue','violet']
    xs=[random.randint(0,10000)/100000 for _ in range(numPoints)]
    ys=[random.randint(0,10000) for _ in range(numPoints)]
    labels=[f'label{c}' for c in range(numPoints)]
    colors={}
    for c,label in enumerate(labels):
        colors[label]=rainbow[c%len(rainbow)]
    js.addVar('xs',xs)
    js.addVar('ys',ys)
    js.addVar('labels',labels)
    js.addVar('colors',colors)
    js.addLines(
        f"let canvas=document.getElementById('graph1')",
        f"let chart=new chartScatter(canvas,{{'title':'GRAPH1','numXTicks':20,'numTooltipPoints':1}})",
        f"chart.setData(xs,ys,labels)",
        f"chart.setColors(colors)",
        f"chart.draw()",
        f"chart.setEvents()",
    )
    doc.body.appendChild(VerticalDivider())
    row1=doc.body.row({'style':{'width':'600px','height':'300px'}})
    row1.Canvas({'id':'graph2'})
    js=doc.body.Script()
    numPoints=10000
    xs=[random.randint(1744209442-3600*24*1,1744209442) for _ in range(numPoints)]
    ys=[random.randint(0,10000) for _ in range(numPoints)]
    labels=[f'label{c}' for c in range(numPoints)]
    colors={}
    for c,label in enumerate(labels):
        colors[label]=rainbow[c%len(rainbow)]
    js.addVar('xs2',xs)
    js.addVar('ys2',ys)
    js.addVar('labels2',labels)
    js.addVar('colors2',colors)
    js.addLines(
        f"canvas=document.getElementById('graph2')",
        f"chart=new chartScatterTime(canvas,{{'title':'GRAPH2','numXTicks':10}})",
        f"chart.setData(xs2,ys2,labels2)",
        f"chart.setColors(colors2)",
        f"chart.draw()",
        f"chart.setEvents()",
    )
    doc.body.appendChild(VerticalDivider())
    row1=doc.body.row({'style':{'width':'600px','height':'300px'}})
    row1.Canvas({'id':'graph3'})
    js=doc.body.Script()
    numPoints=100
    xs=list(range(numPoints))
    ys1=[random.randint(0,numPoints) for _ in range(numPoints)]
    ys2=[random.randint(0,numPoints) for _ in range(numPoints)]
    ys3=[random.randint(0,numPoints) for _ in range(numPoints)]
    labels=[f'label{c}' for c in range(3)]
    colors={}
    for c,label in enumerate(labels):
        colors[label]=rainbow[c%len(rainbow)]
    js.addVar('xs3',xs)
    js.addVar('ys3a',ys1)
    js.addVar('ys3b',ys2)
    js.addVar('ys3c',ys3)
    js.addVar('labels3',labels)
    js.addVar('colors3',colors)
    js.addLines(
        f"canvas=document.getElementById('graph3')",
        f"chart=new chartLine(canvas,{{'title':'GRAPH3'}})",
        f"chart.addDataSet(xs3,ys3a,labels3[0])",
        f"chart.addDataSet(xs3,ys3b,labels3[1])",
        f"chart.addDataSet(xs3,ys3c,labels3[2])",
        f"chart.setColors(colors3)",
        f"chart.draw()",
        f"chart.setEvents()",
    )
    docstr=doc.str().encode(encoding='raw_unicode_escape').replace(b'\\u',b'&#x').replace(b'\\U',b'&#x')
    print(round(len(docstr)/1024/1024,1),'MB')
    return Response(docstr,media_type='text/html')

if __name__=='__main__':
    uvicorn.run(app,port=5001)